unit OpenGLLight;

interface

uses
  OpenGL;

type

  TLightParameters = packed record
    r, g, b, a: glFloat;
  end;

  TLightPosition = packed record
    x, y, z, w: glFloat;
  end;

  TGLLight = class
  private
    fAmbient, fDiffuse, fSpecular: TLightParameters;
    fPosition: TLightPosition;
  public
    procedure setAmbient(r, g, b, a: double);
    procedure setDiffuse(r, g, b, a: double);
    procedure setSpecular(r, g, b, a: double);
    procedure setPosition(x, y, z, w: double);
    constructor Create;
  published
    property ambient: TLightParameters read fAmbient;
    property diffuse: TLightParameters read fDiffuse;
    property specular: TLightParameters read fSpecular;
    property position: TLightPosition read fPosition;
  end;
  pTGLLight = ^TGLLight;

implementation

{ TGLLight }

procedure TGLLight.setAmbient(r, g, b, a: double);
begin
  fAmbient.r := r;
  fAmbient.g := g;
  fAmbient.b := b;
  fAmbient.a := a;
end;

procedure TGLLight.setDiffuse(r, g, b, a: double);
begin
  fDiffuse.r := r;
  fDiffuse.g := g;
  fDiffuse.b := b;
  fDiffuse.a := a;
end;

procedure TGLLight.setSpecular(r, g, b, a: double);
begin
  fSpecular.r := r;
  fSpecular.g := g;
  fSpecular.b := b;
  fSpecular.a := a;
end;

procedure TGLLight.setPosition(x, y, z, w: double);
begin
  fPosition.x := x;
  fPosition.y := y;
  fPosition.z := z;
  fPosition.w := w;
end;

constructor TGLLight.Create;
begin
  fAmbient.a := 1;
  fDiffuse.a := 1;
  fSpecular.a := 1;
end;

end.
