unit OpenGLManipulator;

interface

uses
  Classes, Controls, Types, OpenGL, OpenGLCamera, OpenGLTypes, Forms, ExtCtrls;

type

  TGLManipulator = class
  protected
    camera: TGLCamera;
//    scene: TGLScene;
    form: TForm;
    drawArea: TControl;
    //useful fields to be used by descendant classes
    RotX, RotY, RotZ: Double;
    //Used in Manipulate procedure to set rotation center
    TransX, TransY, TransZ: Double;
    //fields needed to reset camera's position
    initEye, initLookAt: TPoint3;
    //used when drawing on form, but with gaps from one or more sides
    leftGap, topGap, rightGap, bottomGap: Integer;
  public
    //keyboard and mouse events handling, return value inicates whether drawing area should be repainted
    function OnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState): TGLEventResult; virtual;
    function OnKeyPress(Sender: TObject; var Key: Char): TGLEventResult; virtual;
    function OnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState): TGLEventResult; virtual;
    function OnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer): TGLEventResult; virtual;
    function OnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer): TGLEventResult; virtual;
    function OnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer): TGLEventResult; virtual;
    function OnMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean): TGLEventResult; virtual;
    function OnMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean): TGLEventResult; virtual;
    function OnMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean): TGLEventResult; virtual;
    //translations and rotations needed to be done during drawing, routine called by TGLScene.DrawObjects
    procedure Manipulate; virtual; 
    //other routines
    procedure SetCamera(camera: TGLCamera);
    //constructor
    constructor Create(form: TForm; drawArea: TControl); virtual;
    //procedure which can be used to set initial state of window, etc. by manipulator
    procedure Initialize; virtual;
    //set camera on it's initial position, and set all rotation angles to 0
    procedure Reset; virtual;
    procedure SetRotationCenter(X, Y, Z: Double);
    procedure SetGaps(left, top, right, bottom: Integer); virtual;
    destructor Destroy; override;
  end;

implementation

{ TGLManipulator }

constructor TGLManipulator.Create(form: TForm; drawArea: TControl);
begin
  self.form := form;
  self.drawArea := drawArea;
  if (drawArea <> form) and (drawArea.ClassName = 'TPanel') then
  begin
    (drawArea as TPanel).OnMouseMove := form.OnMouseMove;
    (drawArea as TPanel).OnMouseDown := form.OnMouseDown;
    (drawArea as TPanel).OnMouseUp := form.OnMouseUp;
  end;
end;

destructor TGLManipulator.Destroy;
begin
  if initEye <> nil then
    initEye.Destroy;
  if initLookAt <> nil then
    initLookAt.Destroy;  
  inherited;
end;

procedure TGLManipulator.Initialize;
begin
  //do nothing, but can be overrided in descendant manipulators classes
end;

procedure TGLManipulator.Manipulate;
begin
  if (TransX <>0) or (TransY <> 0) or (TransZ <> 0) then
    glTranslate(TransX, TransY, TransZ);
  if RotX <> 0 then
    glRotate(RotX, 1, 0, 0);
  if RotY <> 0 then
    glRotate(RotY, 0, 1, 0);//good for the dinosaur, think!
  if RotZ <> 0 then
    glRotate(RotZ, 0, 0, 1);//good for the dinosaur, think!
  if (TransX <>0) or (TransY <> 0) or (TransZ <> 0) then
    glTranslate(-TransX, -TransY, -TransZ);
end;

function TGLManipulator.OnKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnKeyPress(Sender: TObject;
  var Key: Char): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnMouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean): TGLEventResult;
begin
  Result := glerNotHandled;
end;

function TGLManipulator.OnMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean): TGLEventResult;
begin
  Result := glerNotHandled;
end;

procedure TGLManipulator.Reset;
var
  vecUp: TVector3;
begin
  RotX := 0;
  RotY := 0;
  RotZ := 0;
  vecUp := TVector3.Create(0, 1, 0);
  camera.SetPosition(initEye, initLookAt, vecUp);
end;

procedure TGLManipulator.SetCamera(camera: TGLCamera);
begin
  self.camera := camera;
  self.initEye := TPoint3.Create;
  self.initLookAt := TPoint3.Create;
  camera.GetPosition(self.initEye, self.initLookAt);
end;

procedure TGLManipulator.SetGaps(left, top, right, bottom: Integer);
begin
  self.leftGap := left;
  self.topGap := top;
  self.rightGap := right;
  self.bottomGap := bottom;
end;

procedure TGLManipulator.SetRotationCenter(X, Y, Z: Double);
begin
  TransX := X;
  TransY := Y;
  TransZ := Z;
end;

end.
