unit OpenGLManipulatorForObjects;

interface

uses
  Classes, Controls, Types, Forms, OpenGL, OpenGLTypes, Dialogs, SysUtils,
  OpenGLScene, OpenGLManipulator, OpenGLCamera, OpenGLInterfaceTools, OpenGLTools;

type

  TGLManipulatorForObjects = class(TGLManipulator)
  private
    manipulatingMode: Boolean;
    startX, startY: Integer;
    drawAreaCursor: TCursor;
    mouseLeftPressed: Boolean;
    //interface tools
    wmSwitch: TGLWindowModeSwitchTool;
    mouseClip: TGLMouseAreaClipTool;
    cTool: TComponentTool;
    procedure SetManipulatingMode;
    procedure SetWorkingMode;
  public
    constructor Create(form: TForm; drawArea: TControl); override;
    //keyboard and mouse events handling, return value inicates whether drawing area should be repainted
    function OnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState): TGLEventResult; override;
    function OnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer): TGLEventResult; override;
    function OnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer): TGLEventResult; override;
    function OnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer): TGLEventResult; override;
    function OnMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean): TGLEventResult; override;
    //translations and rotations needed to be done during drawing, routine called by TGLScene.DrawObjects
    procedure Manipulate; override;
    //procedure to set some initial parameters, or performe some initial procedures
    procedure Initialize; override;
    procedure SetGaps(left, top, right, bottom: Integer); override;
    destructor Destroy; override;
  end;

implementation

{ TGLObjectManipulator }

function TGLManipulatorForObjects.OnKeyDown(Sender: TObject;var Key: Word;
Shift: TShiftState): TGLEventResult;
begin
  Result := glerRepaintNeeded;
  case Key of
//  37://left
//    camera.Yaw(-1);
//  39://right
//    camera.Yaw(1);
  38://up arrow
    camera.Slide(0, 0, 0.1);
  40://down arrow
    camera.Slide(0, 0, -0.1);
  70://f - fullscreen mode
    begin
      //set fullscreen window mode
      wmSwitch.SwitchFullScreen;
      if manipulatingMode then mouseClip.UpdateClipArea;
    end;
  78://n - normal mode
    begin
      //set normal window mode
      wmSwitch.SwitchNormal;
      if manipulatingMode then mouseClip.UpdateClipArea;
    end;
  77://m - mode
    begin
      //set opposite window mode
      wmSwitch.SwitchOpposite;
      if manipulatingMode then mouseClip.UpdateClipArea;
    end;
  else
//    ShowMessage(IntToStr(Key));
    Result := glerNotHandled;
  end;
end;

function TGLManipulatorForObjects.OnMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer): TGLEventResult;
begin
  if Button = mbRight then
  begin
    if manipulatingMode then
      SetWorkingMode
    else
      SetManipulatingMode;
    Result := glerHandledNoRepaint;
  end
  else if manipulatingMode then
  begin
    if Button = mbLeft then
      mouseLeftPressed := true;
    Result := glerHandledNoRepaint;
  end
  else
    Result := glerNotHandled;
end;

function TGLManipulatorForObjects.OnMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer): TGLEventResult;
begin
  Result := glerNotHandled;
  if manipulatingMode then
    if mouseLeftPressed then
    begin
      camera.Slide(-(X-startX)/300, (Y-startY)/300, 0);
      mouseClip.SetCursor(X, Y, startX, startY);
      Result := glerRepaintNeeded;
    end
    else
    begin
      rotY := rotY+(X-startX)/2;
      rotX := rotX+(Y-startY)/2;
      if (rotY > 360) or (rotY < -360) then rotY := 0;
      if (rotX > 360) or (rotX < -360) then rotX := 0;
      mouseClip.SetCursor(X, Y, startX, startY);
      Result := glerRepaintNeeded;
    end
end;

function TGLManipulatorForObjects.OnMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer): TGLEventResult;
begin
  if manipulatingMode then
  begin
    if Button = mbLeft then
      mouseLeftPressed := false;
    Result := glerHandledNoRepaint;
  end
  else
    Result := glerNotHandled;
end;

function TGLManipulatorForObjects.OnMouseWheel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean): TGLEventResult;
begin
  if manipulatingMode then
  begin
    camera.Slide(0, 0, WheelDelta/1400);
    Result := glerRepaintNeeded;
  end
  else
    Result := glerNotHandled;
end;

procedure TGLManipulatorForObjects.Manipulate;
begin
  inherited;
{
  if RotX <> 0 then
  begin
    glRotate(RotX, 1, 0, 0);
  end;
  if RotY <> 0 then
  begin
    glRotate(RotY, 0, 1, 0);//good for the dinosaur, think!
    //glRotate(RotY, 0, 0, 1);
  end;
}  
end;

constructor TGLManipulatorForObjects.Create(form: TForm; drawArea: TControl);
begin
  inherited;
  wmSwitch := TGLWindowModeSwitchTool.Create(form);
  mouseClip := TGLMouseAreaClipTool.Create(drawArea);
  cTool := TComponentTool.Create(form);
end;

procedure TGLManipulatorForObjects.Initialize;
begin
  inherited;
  //SetManipulatingMode;
end;

procedure TGLManipulatorForObjects.SetManipulatingMode;
begin
  if not manipulatingMode then
  begin
    drawAreaCursor := drawArea.Cursor;
    drawArea.Cursor := crNone;
    mouseClip.Clip(startX, startY);
    cTool.SwitchFocusedControlDisabledEnabled;
//    TComponentTool.SetFocusedControl(form, false);
    manipulatingMode := true;
  end;
end;

procedure TGLManipulatorForObjects.SetWorkingMode;
begin
  if manipulatingMode then
  begin
    drawArea.Cursor := drawAreaCursor;
    mouseClip.Release(startX, startY);
    cTool.SwitchFocusedControlDisabledEnabled;
//    TComponentTool.SetFocusedControl(form, true);
    manipulatingMode := false;
  end;
end;

procedure TGLManipulatorForObjects.SetGaps(left, top, right, bottom: Integer);
begin
  inherited;
  self.mouseClip.SetGaps(left, top, right, bottom);
end;

destructor TGLManipulatorForObjects.Destroy;
begin
  self.wmSwitch.Destroy;
  self.mouseClip.Destroy;
  inherited;
end;

end.
