unit OpenGLObj;

interface

uses
  OpenGL, OpenGLObject, OpenGLTypes, SysUtils, OpenGLWritableObject;

type

  TObjFaceItem = class
  private
    vNum, vtNum, vnNum: Integer;
  public
    constructor Create(vNum, vtNum, vnNum: Integer);
  end;

  TObjFace = class
  private
    Items: array of TObjFaceItem;
    iCount: Integer;
  public
    constructor Create(iNum: Integer);
  end;

  TObjGroup = class
    name: String;
    material: String; //?? maybe it's a file name, maybe will be better to set
                      //some material properties, like TGLMaterial object (yet to be implemented)
    fMinNo, fMaxNo: Integer;
    constructor Create;
  end;

  TGLObj = class(TGLObject)
  private
    //data structures
    v: array of TPoint3;
    vCount: Integer;
    vn: array of TVector3;
    vnCount: Integer;
    vt: array of TPoint3; //maybe it should be another object type, but we don't really know how it works
    vtCount: Integer;
    f: array of TObjFace;
    fCount: Integer;
    g: array of TObjGroup;
    gCount: Integer;
    xMin, xMax, yMin, yMax, zMin, zMax:double;  // vertices values x, y, and z range
    sxMin, sxMax, syMin, syMax, szMin, szMax:double; // screen values ranges
    currentIndex: Integer;
    function s2x(x: Double): Double;
    function s2y(y: Double): Double;
    function s2z(z: Double): Double;
    procedure DrawFace(faceNo: Integer);
  public
    constructor Create(vNo, vnNo, vtNo, fNo, gNo: Integer);
    // all vertices, vns, groups, faces, etc. have to be added in correct
    // order (order of reading lines from the file)
    procedure AddVertex(x, y, z: double);
    procedure AddVectorNormal(a, b, c: double);
    procedure AddVTexture(u, v, w: Double);
    procedure AddGroup(name: String);
    procedure SetGroupMaterial(material: String); //set material for the last added group
    procedure AddFace(itemsNo: Integer); // adds face to the last added group
    procedure AddFaceItem(vNum, vtNum, vnNum: Integer); //adds item to the last added face
    function GetFacesCount: Integer;
    function GetFirstFace: TGLFace;
    function GetNextFace: TGLFace;
    // implementation of abstract procedure inherited from TGLObject
    procedure Draw; override;
  end;

implementation

{ TGLObjObject }

procedure TGLObj.AddFace(itemsNo: Integer);
var
  face: TObjFace;
begin
  face := TObjFace.Create(itemsNo);
  self.f[fCount] := face;
  Inc(fCount);
end;

procedure TGLObj.AddFaceItem(vNum, vtNum, vnNum: Integer);
var
  fItem: TObjFaceItem;
begin
  fItem := TObjFaceItem.Create(vNum, vtNum, vnNum);
  f[fCount-1].Items[f[fCount-1].iCount] := fItem;
  Inc(f[fCount-1].iCount);
end;

procedure TGLObj.AddGroup(name: String);
var
  group: TObjGroup;
begin
  group := TObjGroup.Create;
  group.name := name;
  group.fMinNo := fCount;
  g[gCount] := group;
  if gCount>0 then
    g[gCount-1].fMaxNo := fCount-1;
  Inc(gCount);
  //Important. When checking if face belongs to group remember to check if
  //g.fMaxNo>=g.fMinNo (if not - empty group),
  //but if g.fMaxNo = -1 then group maybe contains all faces > fMinNo
end;

procedure TGLObj.AddVectorNormal(a, b, c: double);
var
  vect: TVector3;
begin
  vect := TVector3.Create(a, b, c);
  vn[vnCount] := vect;
  Inc(vnCount);
end;

procedure TGLObj.AddVertex(x, y, z: double);
var
  vert: TPoint3;
begin
  vert := TPoint3.Create(x, y, z);
  v[vCount] := vert;
  Inc(vCount);
  if x<xMin then xMin := x;
  if x>xMax then xMax := x;
  if y<yMin then yMin := y;
  if y>yMax then yMax := y;
  if z<zMin then zMin := z;
  if z>zMax then zMax := z;
  syMin := -((yMax-yMin)*(sxMax-sxMin)/(xMax-xMin))/2;
  syMax := ((yMax-yMin)*(sxMax-sxMin)/(xMax-xMin))/2;
  szMax := (zMax-zMin)*(sxMax-sxMin)/(xMax-xMin);
end;

procedure TGLObj.AddVTexture(u, v, w: Double);
var
  vTex: TPoint3;
begin
  vTex := TPoint3.Create(u, v, w);
  vt[vtCount] := vTex;
  Inc(vtCount);
end;

constructor TGLObj.Create(vNo, vnNo, vtNo, fNo, gNo: Integer);
begin
  SetLength(v, vNo);
  SetLength(vn, vnNo);
  SetLength(vt, vtNo);
  SetLength(f, fNo);
  SetLength(g, gNo);
  vCount := 0;
  vnCount := 0;
  vtCount := 0;
  fCount := 0;
  gCount := 0;
  xMin := 100000;
  xMax := -100000;
  yMin := 100000;
  yMax := -100000;
  zMin := 100000;
  zMax := -100000;
  sxMin := -1;
  sxMax := 1;
  szMin := 0;
end;

procedure TGLObj.Draw;
var
  i, j: Integer;
  group: TObjGroup;
begin
  if gCount>0 then //there are groups in obj
  begin
    group := g[0];
    // draw all faces before the first group (not contained with any group) if there are any
    for i:=0 to group.fMinNo-1 do
      DrawFace(i);
    // draw all groups of faces
    for j:=0 to gCount-1 do
    begin
      group := g[j];
      //set material for actuall group
      //...
      //draw all faces contained into that group
      if j=gCount-1 then //last group
        for i:=group.fMinNo to fCount-1 do
          DrawFace(i)
      else // not the last group
        for i:=group.fMinNo to group.fMaxNo do
          DrawFace(i);
    end;
  end
  else // there are no groups in obj
    for i:=0 to fCount-1 do
      DrawFace(i);
end;

{ TObjFace }

constructor TObjFace.Create(iNum: Integer);
begin
  if iNum>0 then
    SetLength(self.Items, iNum);
  iCount := 0;
end;

procedure TGLObj.DrawFace(faceNo: Integer);
var
  i: Integer;
  face: TObjFace;
  fItem: TObjFaceItem;
  vert: TPoint3;
  vect: TVector3;
begin
  face := f[faceNo];
  glBegin(GL_TRIANGLE_FAN);
  for i:=0 to face.iCount-1 do
  begin
    fItem := face.Items[i];
    //check if there is vector, so need to add starting conditions, like vNum := -1;
    if (fItem.vnNum <> -1) then
    begin
      vect := vn[fItem.vnNum-1];
      if vect <> nil then
        glNormal3f(vect.x, vect.y, vect.z);
    end;
    if (fItem.vNum <> -1) then
    begin
      vert := v[fItem.vNum-1];
      if vert <> nil then
        glVertex3f( s2x(vert.x), s2y(vert.y), s2z(vert.z));
    end;
  end;
  glEnd;
end;

function TGLObj.GetFacesCount: Integer;
begin
  Result := fCount;
end;

function TGLObj.GetFirstFace: TGLFace;
var
  i: Integer;
begin
  Result := nil;
  if fCount>0 then
  begin
    //create result object
    Result := TGLFace.Create(f[0].iCount, false);
    //add all vertices of the face to the result
    for i:=0 to f[0].iCount-1 do
      Result.AddVertex(v[f[0].Items[i].vNum].x, v[f[0].Items[i].vNum].y, v[f[0].Items[i].vNum].z);
    currentIndex := 1;
  end;
end;

function TGLObj.GetNextFace: TGLFace;
var
  i: Integer;
begin
  Result := nil;
  if fCount>currentIndex then
  begin
    Result := TGLFace.Create(f[currentIndex].iCount, false);
    //add all vertices of the face to the result
    for i:=0 to f[currentIndex].iCount-1 do
      Result.AddVertex(v[f[currentIndex].Items[i].vNum].x, v[f[currentIndex].Items[i].vNum].y, v[f[currentIndex].Items[i].vNum].z);
    Inc(currentIndex);
  end;
end;

function TGLObj.s2x(x: Double): Double;
begin
  Result := sxMin + (x-xMin)*(sxMax-sxMin)/(xMax-xMin);
end;

function TGLObj.s2y(y: Double): Double;
begin
  Result := syMin + (y-yMin)*(syMax-syMin)/(yMax-yMin);
end;

function TGLObj.s2z(z: Double): Double;
begin
  Result := szMin + (z-zMin)*(szMax-szMin)/(zMax-zMin);
end;

procedure TGLObj.SetGroupMaterial(material: String);
begin
  //example
  g[gCount-1].material := material;
end;

{ TObjFaceItem }

constructor TObjFaceItem.Create(vNum, vtNum, vnNum: Integer);
begin
  self.vNum := vNum;
  self.vtNum := vtNum;
  self.vnNum := vnNum;
end;

{ TObjGroup }

constructor TObjGroup.Create;
begin
  self.fMinNo := -1;
  self.fMaxNo := -1;
end;

end.
