unit OpenGLTools;

interface

uses
  Classes, StdCtrls, Forms, Controls;

type

  stringArray = array of String;

  TStringTool = class
    class function Tokenize(str: String): stringArray; overload;
    class function Tokenize(pattern, str: String): stringArray; overload;
    class procedure Truncate(var str: String);
  end;

  TComponentTool = class
  private
    form: TForm;
    activeControl: TWinControl;
    enabled: Boolean;
    procedure SetAllComponents(component: TComponent; enable: Boolean);
  public
    constructor Create(form: TForm);
    procedure SwitchFocusedControlDisabledEnabled;
  end;

implementation

{ TStringTool }

class function TStringTool.Tokenize(str: String): stringArray;
begin
  Result := Tokenize(' ', str);
end;

class function TStringTool.Tokenize(pattern, str: String): stringArray;
var
  subString: String;
  position, i: Integer;
  tokensNum: Integer;
begin
  Truncate(str);
  if Length(str) > 0 then
  begin
    subString := str;
    tokensNum := 0;
    while (true) do
    begin
      position := Pos(pattern, subString);
      if position = 0 then
      begin
        Inc(tokensNum);
        break;
      end;
      Delete(subString, 1, position+Length(pattern)-1);
      Inc(tokensNum);
      Truncate(subString);
      if Length(subString) = 0 then break;
    end;
    SetLength(Result, tokensNum);
    subString := str;
    for i:=0 to tokensNum-1 do
    begin
      position := Pos(pattern, subString);
      if position > 0 then
      begin
        Result[i] := Copy(subString, 1, position-1);
        Delete(subString, 1, position+Length(pattern)-1);
        Truncate(subString)
      end
      else
        Result[i] := subString;
    end;
  end
  else
    Result := nil;
end;

class procedure TStringTool.Truncate(var str: String);
var
  position: Integer;
begin
  while (true) do
  begin
    position := Pos(' ', str);
    if position = 1 then
      Delete(str, 1, 1)
    else if (position <> 0) and (position = Length(str)) then
      Delete(str, position, position)
    else break;
  end;
end;

{ TComponentTool }

constructor TComponentTool.Create(form: TForm);
begin
  self.form := form;
  self.enabled := true;
end;

procedure TComponentTool.SetAllComponents(component: TComponent; enable: Boolean);
var
  i: Integer;
  comp: TComponent;
begin
  for i:=0 to component.ComponentCount-1 do
  begin
    comp := component.Components[i];
    if comp.ClassName = 'TEdit' then
      (comp as TEdit).Enabled := enable
    else if comp.ClassName = 'TComboBox' then
      (comp as TComboBox).Enabled := enable
    else if comp.ClassName = 'TPanel' then
      SetAllComponents(comp, enable);
  end;
end;

procedure TComponentTool.SwitchFocusedControlDisabledEnabled;
begin
  if enabled then
  begin
    activeControl := form.ActiveControl;
    if activeControl <> nil then
    begin
      if activeControl.ClassNameIs('TEdit') or activeControl.ClassNameIs('TComboBox') then
        activeControl.Enabled := false;
      enabled := false;
    end;  
  end
  else
  begin
    activeControl.Enabled := true;
    form.SetFocusedControl(activeControl);
    enabled := true;
  end;
end;

end.
