unit OpenGLTypes;

interface

uses
  OpenGL, OpenGLObject;

type

  TGLEventResult = (glerHandledNoRepaint, glerRepaintNeeded, glerNotHandled);

  TGLAxes = class(TGLObject)
    procedure Draw; override;
  end;

  TLightDef = packed record
    red,
    green,
    blue,
    alpha: glFloat;
  end;

  TPosition = packed record
    x,
    y,
    z,
    w: glFloat;
  end;

  PHit = ^THit;
  THit = packed record
    ncount,
    dnear,
    dfar:GLuint;
    names:array [1..32] of GLuint;
  end;

  TPoint3 = class
    x, y, z: double;
    procedure setCoordinates(dx, dy, dz: double); overload;
    procedure setCoordinates(p: TPoint3); overload;
    constructor Create; overload;
    constructor Create(xx, yy, zz: double); overload;
  end;

  TVector3 = class
    x, y, z: double;
    procedure setVector(dx, dy, dz: double); overload;
    procedure setVector(v: TVector3); overload;
    procedure Normalize;
    constructor Create; overload;
    constructor Create(xx, yy, zz: double); overload;
    constructor Create(v: TVector3); overload;
    function Cross(v: TVector3): TVector3; overload;
    procedure Cross(v, vResult: TVector3); overload;
    function Dot(v: TVector3): double;
    function Length: double;
  end;

implementation

{ TPoint3 }

constructor TPoint3.Create;
begin
  self.x := 0;
  self.y := 0;
  self.z := 0;
end;

constructor TPoint3.Create(xx, yy, zz: double);
begin
  self.x := xx;
  self.y := yy;
  self.z := zz;
end;

procedure TPoint3.setCoordinates(p: TPoint3);
begin
  self.x := p.x;
  self.y := p.y;
  self.z := p.z;
end;

procedure TPoint3.setCoordinates(dx, dy, dz: double);
begin
  self.x := dx;
  self.y := dy;
  self.z := dz;
end;

{ TVector3 }

constructor TVector3.Create;
begin
  self.x := 0;
  self.y := 0;
  self.z := 0;
end;

constructor TVector3.Create(xx, yy, zz: double);
begin
  self.x := xx;
  self.y := yy;
  self.z := zz;
end;

constructor TVector3.Create(v: TVector3);
begin
  self.x := v.x;
  self.y := v.y;
  self.z := v.z;
end;

//return self x v (cross product)
function TVector3.Cross(v: TVector3): TVector3;
begin
  result := TVector3.Create(y * v.z - z * v.y, z * v.x - x * v.z, x * v.y - y * v.x);
end;

procedure TVector3.Cross(v, vResult: TVector3);
begin
  if vResult = nil then
    vResult.setVector(y * v.z - z * v.y, z * v.x - x * v.z, x * v.y - y * v.x);
end;

//return self * v (dot product)
function TVector3.Dot(v: TVector3): double;
begin
  result := x * v.x + y * v.y + z * v.z;
end;

function TVector3.Length: double;
begin
  Result := sqrt(x*x + y*y + z*z);
end;

procedure TVector3.Normalize;
var
  len: double;
begin
  len := sqr(x) + sqr(y) + sqr(z);
  if len > 0.0000001 then //if it's not a zero vector
  begin
    len := sqrt(len);
    x := x/len;
    y := y/len;
    z := z/len;
  end;
end;

procedure TVector3.setVector(v: TVector3);
begin
  self.x := v.x;
  self.y := v.y;
  self.z := v.z;
end;

procedure TVector3.setVector(dx, dy, dz: double);
begin
  self.x := dx;
  self.y := dy;
  self.z := dz;
end;

{ TGLAxes }

procedure TGLAxes.Draw;
begin
   glBegin(GL_LINES);
   glColor3f(1,0,0);
      glVertex3f(0,0,0);
      glVertex3f(1000,0,0);
   glColor3f(0,1,0);
      glVertex3f(0,0,0);
      glVertex3f(0,1000,0);
   glColor3f(0,0,1);
      glVertex3f(0,0,0);
      glVertex3f(0,0,1000);
   glEnd;
end;

end.
