//*****************************************************************************
// File:        PartitionSurface.pas
//
// Classe:      TPartitionSurface
//
// Description: TPartitionSurface computes the partition surface for a data set
//              in 3d. Is scans each edge and if their attributes are different
//              then the voronoi plane associated with the edge is stored. It's
//              like extracting a boundary.
//
// Note:
//
//*****************************************************************************
// Last version: 25-06-2002
//*****************************************************************************

unit PartitionSurface;

interface

uses
  Mesh,
  Point,
  Edge,
  Tetra,
  Classes;

type
  TPartitionSurface = class
  private
    m_mesh:     TMesh;
    function    ComputeSurface : TList;
  public
    constructor Create(mesh: TMesh);
    procedure   DrawPartitionSurface;
  end;


implementation

{ TSplitSurface }

constructor TPartitionSurface.Create(mesh: TMesh);
begin
  m_mesh := mesh;
end;


function TPartitionSurface.ComputeSurface: TList;
var
  lstTe: TList;
  lstEdges: TList;
  i: integer;
  j: integer;
  k: integer;
  l: integer;
  te: TTetra;
  a: TPoint;
  b: TPoint;
  e: TEdge;
  found: boolean;
begin
  e := TEdge.Create;
  lstEdges := TList.Create;
  lstTe := m_mesh.GetListTetra;
  for i := 0 to (lstTe.Count - 1) do  //-- for each tetrahedron in the mesh
  begin
    te := lstTe[i];
    //-- compute the intersection between the surface and each edge of the tetra
    for j := 1 to 3 do
    begin
      a := te.GetP(j);
      for k := j + 1 to 4 do
      begin
        b := te.GetP(k);
        if ( (a.data <> b.data) and (a.data <> 0) and (b.data <> 0) ) then
        begin
          e.Reset(a, b, te);
          //-- only store an edge once in the list
          found := false;
          for l := 0 to (lstEdges.Count - 1) do
          begin
            if (e.Compare(lstEdges[l]) = true) then
            begin
              found := true;
            end;
          end;
          //-- if it's not in the list, add it
          if (found = false) then
          begin
            lstEdges.Add(TEdge.Create(a, b, te));
          end;
        end;
      end;
    end;
  end;
  Result := lstEdges;
end;


procedure TPartitionSurface.DrawPartitionSurface;
var
  lstEdges: TList;
  i: integer;
begin
  lstEdges := self.ComputeSurface;
  m_mesh.bDrawPartition := true;
  m_mesh.SetPartitionSurface(lstEdges);

  //-- free the list of edges
  for i := 0 to (lstEdges.Count - 1) do
  begin
    TEdge(lstEdges[i]).Free;
  end;
  lstEdges.Free;
end;



end.
